# This Build.PL for Dist-Zilla-Plugin-DynamicPrereqs-Meta was generated by Dist::Zilla::Plugin::ModuleBuildTiny 0.018.
use strict;
use warnings;


use 5.020;
use Module::Build::Tiny 0.034;
Build_PL();

